unit DBBlank;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, Procs, DB, DBrBitBtn, StrUtils, DBTables, RButtons,
  rDBUtils;

type
  TFormDBBlank = class(TForm)
    BitBtn1: TrBitBtn;
    BitBtn2: TDBrBitBtn;
    BitBtn3: TBitBtn;
    GroupBox1: TGroupBox;
    DataSource: TDataSource;
    procedure BitBtn1Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    function FormHelp(Command: Word; Data: Integer;
      var CallHelp: Boolean): Boolean;
  private
    { Private declarations }
    Database: TDatabase;
    NewTrans: Boolean;
    FStartTrans: Boolean;
    FEditMode: Boolean;
    FFormDataSetState: TDataSetState;
    FGender: TGenders;
    function GetDataSet: TDataSet;
    procedure SetDataSet(Value: TDataSet);
    procedure SetStartTrans(Value: Boolean);
  public
    { Public declarations }
    ExtComps: array [0..20] of TComponent;
    ConfirmCancel: Boolean;
    constructor DBCreate(AOwner: TComponent; ADataSet: TDataSet; AGender: TGenders;
                         AExtComps: array of TComponent);
    property EditMode: Boolean read FEditMode write FEditMode;
    property FormDataSet: TDataSet read GetDataSet write SetDataSet;
    property FormDataSetState: TDataSetState read FFormDataSetState;
    property StartTrans: Boolean read FStartTrans write SetStartTrans;
    function Editing: Boolean;
    procedure RefreshDBState;
  end;

var
  FormDBBlank: TFormDBBlank;

implementation

{$R *.DFM}

constructor TFormDBBlank.DBCreate;
var i: Integer;
begin
  FGender := AGender;
  ConfirmCancel := True;
  for i:=0 to High(AExtComps) do  ExtComps[i] := AExtComps[i];
  inherited Create(AOwner);
  NewTrans := False;
  if ADataSet <> nil then FormDataSet := ADataSet
  else RefreshDBState;
end;

procedure TFormDBBlank.BitBtn1Click(Sender: TObject);
begin
  if Editing then begin
    FormDataSet.Post;
    if NewTrans then
      with Database do if InTransaction then Commit;
    if (FFormDataSetState = dsInsert) or (FormDataSet is TQuery) then
      if (FormDataSet is TQuery) and TQuery(FormDataSet).RequestLive then begin
        FormDataSet.Close;
        FormDataSet.Open;
      end else
        DoAction(FormDataSet,nil,tbaRefresh);
  end;
  ModalResult := mrOK;
end;

function TFormDBBlank.GetDataSet;
begin
  Result := DataSource.DataSet;
end;

procedure TFormDBBlank.SetDataSet;
begin
  //      -  RefreshDBState
  DataSource.DataSet := Value;
  FEditMode := Editing;
  FFormDataSetState := Value.State;
  if (DataSource.DataSet<>nil) and (DataSource.State=dsInsert) then
    Caption := ChangeEndStr(Caption,WIN_CAP_DELIM,NewStrings[FGender])
  else
    Caption := ChangeEndStr(Caption,WIN_CAP_DELIM,'');
  if NewTrans then
    with Database do if InTransaction then Commit;
  if Value <> nil then begin
    Database := TDBDataSet(Value).Database;
    NewTrans := False;
    if FStartTrans and FEditMode then
      with Database do if not InTransaction then begin
        StartTransaction;
        NewTrans := True;
      end;
  end;
  if FEditMode then begin
    Caption := ReplaceStr(Caption,' [View]','');
    with BitBtn1 do begin
      GlyphResource := 'BBOK';
      NumGlyphs := 2;
      if not NewTrans and Assigned(Database) and Database.InTransaction then begin
        Caption := 'OK';
        HelpContext := 20104;
      end else begin
        Caption := 'Save';
        HelpContext := 20101;
      end;
    end;
  end else begin
    Caption := ChangeEndStr(Caption,' [','View]');
    with BitBtn1 do begin
      GlyphResource := 'EXIT_BTN';
      NumGlyphs := 1;
      Caption := '&Close';
      HelpContext := 20102;
    end;
  end;
  BitBtn1.Cancel := not FEditMode;
end;

procedure TFormDBBlank.SetStartTrans;
begin
  if FStartTrans <> Value then begin
    if Value and FEditMode then
      with Database do if not InTransaction then begin
        StartTransaction;
        NewTrans := True;
      end;
    FStartTrans := Value;
  end;
end;

function TFormDBBlank.Editing;
begin
  Result := (FormDataSet<>nil) and (FormDataSet.State in [dsEdit,dsInsert]);
end;

procedure TFormDBBlank.RefreshDBState;
begin
  SetDataSet(FormDataSet);
end;

procedure TFormDBBlank.FormDestroy(Sender: TObject);
begin
  if Editing then FormDataSet.Cancel;
end;

procedure TFormDBBlank.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key = VK_RETURN) and (ssCtrl in Shift) then BitBtn1.Click;
end;

procedure TFormDBBlank.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  if (ModalResult=mrCancel) and FormDataSet.Modified and ConfirmCancel and
     not Procs.ConfirmCancel('Cancel changes') then begin
    Action := caNone;
    exit;
  end;
  if NewTrans then
    with Database do
      if InTransaction and (ModalResult <> mrOK) then Rollback;
end;

function TFormDBBlank.FormHelp(Command: Word; Data: Integer;
  var CallHelp: Boolean): Boolean;
begin
  CallHelp := not ((Command = HELP_CONTEXTPOPUP) and (Data = HelpContext));
  Result := True;
end;

end.
